@@ -1,10 +1,10 @@
 
 
-#' Theme a Huxtable
+#' Theme a huxtable
 #'
 #' These functions quickly set default styles for a huxtable.
 #'
-#' \code{theme_minimal} is a minimal theme with a simple line under headers.
+#' \code{theme_basic} is a simple theme with a simple line under headers.
 #'
 #' \code{theme_striped} uses different backgrounds for alternate rows, and for headers.
 #'
@@ -26,10 +26,10 @@ NULL
 
 #' @export
 #' @rdname themes
-theme_minimal <- function (ht, header_row = TRUE, header_col = TRUE) {
+theme_basic <- function (ht, header_row = TRUE, header_col = TRUE) {
   ht <- set_all_borders(ht, 1:nrow(ht), 1:ncol(ht), 0)
-  if (header_row) bottom_border(ht)[1,] <- 1
-  if (header_col) right_border(ht)[,1] <- 1
+  if (header_row) bottom_border(ht)[1, ] <- 1
+  if (header_col) right_border(ht)[, 1] <- 1
 
   ht
 }
@@ -40,17 +40,19 @@ theme_minimal <- function (ht, header_row = TRUE, header_col = TRUE) {
 #' @param stripe Background colour for alternate rows
 theme_striped <- function (ht, stripe = grey(.9), header_row = TRUE, header_col = TRUE) {
   ht <- set_all_borders(ht, 1:nrow(ht), 1:ncol(ht), 0)
-  background_color(ht)[seq(1, nrow(ht), 2),] <- 'white'
-  if (nrow(ht) >= 2) background_color(ht)[seq(2, nrow(ht), 2),] <- stripe
+  background_color(ht)[seq(1, nrow(ht), 2), ] <- 'white'
+  if (nrow(ht) >= 2) background_color(ht)[seq(2, nrow(ht), 2), ] <- stripe
   if (header_row) {
-    background_color(ht)[1,] <- 'black'
-    text_color(ht)[1,] <- 'white'
-    bold(ht)[1,] <- TRUE
+    background_color(ht)[1, ] <- 'black'
+    text_color(ht)[1, ]       <- 'white'
+    ht <- set_all_border_colors(ht, 1, every(), 'white')
+    bold(ht)[1, ]             <- TRUE
   }
   if (header_col) {
-    background_color(ht)[,1] <- 'black'
-    text_color(ht)[,1] <- 'white'
-    bold(ht)[,1] <- TRUE
+    background_color(ht)[, 1] <- 'black'
+    text_color(ht)[, 1]       <- 'white'
+    ht <- set_all_border_colors(ht, every(), 1, 'white')
+    bold(ht)[, 1]             <- TRUE
   }
 
   ht
@@ -61,14 +63,13 @@ theme_striped <- function (ht, stripe = grey(.9), header_row = TRUE, header_col
 #' @rdname themes
 theme_article <- function(ht, header_row = TRUE, header_col = TRUE) {
   ht <- set_all_borders(ht, 1:nrow(ht), 1:ncol(ht), 0)
-  top_border(ht)[1,] <- 1
-  bottom_border(ht)[nrow(ht),] <- 1
+  top_border(ht)[1, ] <- 1
+  bottom_border(ht)[nrow(ht), ] <- 1
   if (header_row) {
-    bottom_border(ht)[1,] <- 1
-    bold(ht)[1,] <- TRUE
+    bottom_border(ht)[1, ] <- 1
+    bold(ht)[1, ] <- TRUE
   }
-  if (header_col) bold(ht)[,1] <- TRUE
+  if (header_col) bold(ht)[, 1] <- TRUE
 
   ht
 }
-
